@@ -1,8 +1,8 @@
 /*
  *      geany.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: geany.h 3504 2009-01-22 20:33:21Z eht16 $
+ * $Id: geany.h 4929 2010-05-16 18:22:24Z eht16 $
  */
 
 /* Main header - should be included first in all source files.
@@ -39,21 +39,23 @@
 #   define PLAT_GTK 1	/* needed when including ScintillaWidget.h */
 #endif
 
+/* Compatibility for sharing macros between API and core, overridden in plugindata.h */
+#define GEANY(symbol_name) symbol_name
+
 
 /* for detailed description look in the documentation, things are not
  * listed in the documentation should not be changed */
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
 #define GEANY_TEMPLATES_SUBDIR			"templates"
-#define GEANY_CODENAME					"Argon"
+#define GEANY_CODENAME					"Vellam"
 #define GEANY_HOMEPAGE					"http://www.geany.org/"
-#define GEANY_USE_WIN32_DIALOG			0
 #define GEANY_STRING_UNTITLED			_("untitled")
-#define GEANY_WINDOW_MINIMAL_WIDTH		550
-#define GEANY_WINDOW_MINIMAL_HEIGHT		350
+#define GEANY_DEFAULT_DIALOG_HEIGHT		350
 #define GEANY_WINDOW_DEFAULT_WIDTH		900
 #define GEANY_WINDOW_DEFAULT_HEIGHT		600
 
 
+
 /* Common forward declarations */
 typedef struct GeanyDocument GeanyDocument;
 typedef struct GeanyEditor GeanyEditor;
@@ -85,23 +87,12 @@ extern GObject *geany_object;
 extern gboolean	ignore_callback;
 
 
-enum
-{
-	UP,
-	DOWN,
-	LEFT,
-	RIGHT
-};
-
-enum
-{
-	KILOBYTE = 1024,
-	MEGABYTE = (KILOBYTE*1024),
-	GIGABYTE = (MEGABYTE*1024)
-};
-
-
 /* prototype is here so that all files can use it. */
 void geany_debug(gchar const *format, ...) G_GNUC_PRINTF (1, 2);
 
+
+#ifndef G_GNUC_WARN_UNUSED_RESULT
+#define G_GNUC_WARN_UNUSED_RESULT
+#endif
+
 #endif
