@@ -1,8 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 3574 2009-02-13 18:12:49Z eht16 $
+ * $Id: msgwindow.c 4959 2010-05-29 18:36:17Z eht16 $
  */
 
 /*
@@ -42,6 +42,7 @@
 #include "navqueue.h"
 #include "editor.h"
 #include "msgwindow.h"
+#include "keybindings.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -74,12 +75,22 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer user_data);
 
 
-void msgwin_init()
+void msgwin_show_hide_tabs(void)
+{
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.tree_status), interface_prefs.msgwin_status_visible);
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.tree_compiler), interface_prefs.msgwin_compiler_visible);
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.tree_msg), interface_prefs.msgwin_messages_visible);
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.scribble), interface_prefs.msgwin_scribble_visible);
+}
+
+
+void msgwin_init(void)
 {
 	msgwindow.notebook = ui_lookup_widget(main_widgets.window, "notebook_info");
 	msgwindow.tree_status = ui_lookup_widget(main_widgets.window, "treeview3");
 	msgwindow.tree_msg = ui_lookup_widget(main_widgets.window, "treeview4");
 	msgwindow.tree_compiler = ui_lookup_widget(main_widgets.window, "treeview5");
+	msgwindow.scribble = ui_lookup_widget(main_widgets.window, "textview_scribble");
 	msgwindow.find_in_files_dir = NULL;
 
 	prepare_status_tree_view();
@@ -89,15 +100,12 @@ void msgwin_init()
 	msgwindow.popup_msg_menu = create_message_popup_menu(MSG_MESSAGE);
 	msgwindow.popup_compiler_menu = create_message_popup_menu(MSG_COMPILER);
 
-	ui_widget_modify_font_from_string(
-		ui_lookup_widget(main_widgets.window, "textview_scribble"), interface_prefs.msgwin_font);
-
-	g_signal_connect(ui_lookup_widget(main_widgets.window, "textview_scribble"),
-		"populate-popup", G_CALLBACK(on_scribble_populate), NULL);
+	ui_widget_modify_font_from_string(msgwindow.scribble, interface_prefs.msgwin_font);
+	g_signal_connect(msgwindow.scribble, "populate-popup", G_CALLBACK(on_scribble_populate), NULL);
 }
 
 
-void msgwin_finalize()
+void msgwin_finalize(void)
 {
 	g_free(msgwindow.find_in_files_dir);
 }
@@ -105,16 +113,21 @@ void msgwin_finalize()
 
 static gboolean on_msgwin_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event->keyval == GDK_Return ||
-		event->keyval == GDK_ISO_Enter ||
-		event->keyval == GDK_KP_Enter ||
-		event->keyval == GDK_space)
+	if (ui_is_keyval_enter_or_return(event->keyval) || event->keyval == GDK_space)
 	{
-		GdkEventButton button_event;
-
-		button_event.button = 1;
-		button_event.time = event->time;
-		on_msgwin_button_press_event(NULL, &button_event, data);
+		switch (GPOINTER_TO_INT(data))
+		{
+			case MSG_COMPILER:
+			{	/* single click in the compiler treeview */
+				msgwin_goto_compiler_file_line(event->keyval);
+				break;
+			}
+			case MSG_MESSAGE:
+			{	/* single click in the message treeview (results of 'Find usage') */
+				msgwin_goto_messages_file_line(event->keyval);
+				break;
+			}
+		}
 	}
 	return FALSE;
 }
@@ -239,13 +252,14 @@ static const GdkColor *get_color(gint msg_color)
  **/
 void msgwin_compiler_add(gint msg_color, const gchar *format, ...)
 {
-	gchar string[512];
+	gchar *string;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 	msgwin_compiler_add_string(msg_color, string);
+	g_free(string);
 }
 
 
@@ -254,11 +268,17 @@ void msgwin_compiler_add_string(gint msg_color, const gchar *msg)
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	const GdkColor *color = get_color(msg_color);
+	gchar *utf8_msg;
+
+	if (! g_utf8_validate(msg, -1, NULL))
+		utf8_msg = utils_get_utf8_from_locale(msg);
+	else
+		utf8_msg = (gchar *) msg;
 
 	gtk_list_store_append(msgwindow.store_compiler, &iter);
-	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, msg, -1);
+	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, utf8_msg, -1);
 
-	if (ui_prefs.msgwindow_visible)
+	if (ui_prefs.msgwindow_visible && interface_prefs.compiler_tab_autoscroll)
 	{
 		path = gtk_tree_model_get_path(
 			gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_compiler)), &iter);
@@ -266,9 +286,12 @@ void msgwin_compiler_add_string(gint msg_color, const gchar *msg)
 		gtk_tree_path_free(path);
 	}
 
-	/* calling build_menu_update for every build message would be overkill */
-	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, TRUE);
-	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_previous_error, TRUE);
+	/* calling build_menu_update for every build message would be overkill, TODO really should call it once when all done */
+	gtk_widget_set_sensitive(build_get_menu_items(-1)->menu_item[GBG_FIXED][GBF_NEXT_ERROR], TRUE);
+	gtk_widget_set_sensitive(build_get_menu_items(-1)->menu_item[GBG_FIXED][GBF_PREV_ERROR], TRUE);
+
+	if (utf8_msg != msg)
+		g_free(utf8_msg);
 }
 
 
@@ -281,16 +304,18 @@ void msgwin_show_hide(gboolean show)
 		show);
 	ignore_callback = FALSE;
 	ui_widget_show_hide(ui_lookup_widget(main_widgets.window, "scrolledwindow1"), show);
+	/* set the input focus back to the editor */
+	keybindings_send_command(GEANY_KEY_GROUP_FOCUS, GEANY_KEYS_FOCUS_EDITOR);
 }
 
 
 /**
  *  Adds a new message in the messages tab treeview in the messages window.
- *  If @c line and @c doc are set, clicking on this line jumps into the file which is specified
- *  by @c doc into the line specified with @c line.
+ *  If @a line and @a doc are set, clicking on this line jumps into the file which is specified
+ *  by @a doc into the line specified with @a line.
  *
  *  @param msg_color A color to be used for the text. It must be an element of #MsgColors.
- *  @param line The document's line where the message belongs to. Set to -1 to ignore.
+ *  @param line The document's line where the message belongs to. Set to @c -1 to ignore.
  *  @param doc The document. Set to @c NULL to ignore.
  *  @param format @c printf()-style format string.
  *  @param ... Arguments for the @c format string.
@@ -299,14 +324,15 @@ void msgwin_show_hide(gboolean show)
  **/
 void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
 {
-	gchar string[512];
+	gchar *string;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 
 	msgwin_msg_add_string(msg_color, line, doc, string);
+	g_free(string);
 }
 
 
@@ -316,26 +342,37 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 	GtkTreeIter iter;
 	const GdkColor *color = get_color(msg_color);
 	gchar *tmp;
+	gsize len;
+	gchar *utf8_msg;
 
-	if (! ui_prefs.msgwindow_visible) msgwin_show_hide(TRUE);
+	if (! ui_prefs.msgwindow_visible)
+		msgwin_show_hide(TRUE);
 
 	/* work around a strange problem when adding very long lines(greater than 4000 bytes)
 	 * cut the string to a maximum of 1024 bytes and discard the rest */
 	/* TODO: find the real cause for the display problem / if it is GtkTreeView file a bug report */
-	if (strlen(string) > 1024)
+	len = strlen(string);
+	if (len > 1024)
 		tmp = g_strndup(string, 1024);
 	else
 		tmp = g_strdup(string);
 
+	if (! g_utf8_validate(tmp, -1, NULL))
+		utf8_msg = utils_get_utf8_from_locale(tmp);
+	else
+		utf8_msg = tmp;
+
 	gtk_list_store_append(msgwindow.store_msg, &iter);
-	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, doc, 2, color, 3, tmp, -1);
+	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, doc, 2, color, 3, utf8_msg, -1);
 
 	g_free(tmp);
+	if (utf8_msg != tmp)
+		g_free(utf8_msg);
 }
 
 
 /**
- *  Log a status message *without* setting the status bar.
+ *  Logs a status message *without* setting the status bar.
  *  (Use ui_set_statusbar() to display text on the statusbar)
  *
  *  @param format @c printf()-style format string.
@@ -344,33 +381,32 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 void msgwin_status_add(const gchar *format, ...)
 {
 	GtkTreeIter iter;
-	gchar string[512];
+	gchar *string;
 	gchar *statusmsg, *time_str;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 
 	/* add a timestamp to status messages */
 	time_str = utils_get_current_time_string();
-	if (time_str == NULL)
-		statusmsg = g_strdup(string);
-	else
-		statusmsg = g_strconcat(time_str, ": ", string, NULL);
+	statusmsg = g_strconcat(time_str, ": ", string, NULL);
 	g_free(time_str);
+	g_free(string);
 
 	/* add message to Status window */
 	gtk_list_store_append(msgwindow.store_status, &iter);
 	gtk_list_store_set(msgwindow.store_status, &iter, 0, statusmsg, -1);
 	g_free(statusmsg);
 
-	if (main_status.main_window_realized)
+	if (G_LIKELY(main_status.main_window_realized))
 	{
 		GtkTreePath *path = gtk_tree_model_get_path(gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_status)), &iter);
 
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(msgwindow.tree_status), path, NULL, FALSE, 0.0, 0.0);
-		if (prefs.switch_to_status) gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
+		if (prefs.switch_to_status)
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_STATUS);
 		gtk_tree_path_free(path);
 	}
 }
@@ -419,7 +455,7 @@ on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
 		gchar *string;
 
 		gtk_tree_model_get(model, &iter, str_idx, &string, -1);
-		if (string && *string)
+		if (NZV(string))
 		{
 			gtk_clipboard_set_text(gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", FALSE)),
 				string, -1);
@@ -493,7 +529,7 @@ on_hide_message_window                 (GtkMenuItem     *menuitem,
 
 static GtkWidget *create_message_popup_menu(gint type)
 {
-	GtkWidget *message_popup_menu, *clear, *copy, *image;
+	GtkWidget *message_popup_menu, *clear, *copy, *copy_all, *image;
 
 	message_popup_menu = gtk_menu_new();
 
@@ -503,19 +539,22 @@ static GtkWidget *create_message_popup_menu(gint type)
 	g_signal_connect(clear, "activate",
 		G_CALLBACK(on_message_treeview_clear_activate), GINT_TO_POINTER(type));
 
-	copy = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
-	gtk_widget_show(copy);
-	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
-	g_signal_connect(copy, "activate",
-		G_CALLBACK(on_compiler_treeview_copy_activate), GINT_TO_POINTER(type));
-
-	copy = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
+	copy = gtk_image_menu_item_new_with_mnemonic(_("C_opy"));
 	gtk_widget_show(copy);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
 	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy), image);
 	g_signal_connect(copy, "activate",
+		G_CALLBACK(on_compiler_treeview_copy_activate), GINT_TO_POINTER(type));
+
+	copy_all = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
+	gtk_widget_show(copy_all);
+	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy_all);
+	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	gtk_widget_show(image);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy_all), image);
+	g_signal_connect(copy_all, "activate",
 		G_CALLBACK(on_compiler_treeview_copy_all_activate), GINT_TO_POINTER(type));
 
 	msgwin_menu_add_common_items(GTK_MENU(message_popup_menu));
@@ -572,13 +611,72 @@ find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
 }
 
 
-gboolean msgwin_goto_compiler_file_line()
+static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint keyval)
+{
+	if (!filename || line <= -1)
+		return FALSE;
+
+	/* If the path doesn't exist, try the current document.
+	 * This happens when we receive build messages in the wrong order - after the
+	 * 'Leaving directory' messages */
+	if (!g_file_test(filename, G_FILE_TEST_EXISTS))
+	{
+		gchar *cur_dir = utils_get_current_file_dir_utf8();
+		gchar *name;
+
+		if (cur_dir)
+		{
+			/* we let the user know we couldn't find the parsed filename from the message window */
+			setptr(cur_dir, utils_get_locale_from_utf8(cur_dir));
+			name = g_path_get_basename(filename);
+			setptr(name, g_build_path(G_DIR_SEPARATOR_S, cur_dir, name, NULL));
+			g_free(cur_dir);
+
+			if (g_file_test(name, G_FILE_TEST_EXISTS))
+			{
+				ui_set_statusbar(FALSE, _("Could not find file '%s' - trying the current document path."),
+					filename);
+				filename = name;
+			}
+			else
+				g_free(name);
+		}
+	}
+
+	{
+		gchar *utf8_filename = utils_get_utf8_from_locale(filename);
+		GeanyDocument *doc = document_find_by_filename(utf8_filename);
+		GeanyDocument *old_doc = document_get_current();
+
+		g_free(utf8_filename);
+
+		if (doc == NULL)	/* file not already open */
+			doc = document_open_file(filename, FALSE, NULL, NULL);
+
+		if (doc != NULL)
+		{
+			gboolean ret;
+
+			if (! doc->changed && editor_prefs.use_indicators)	/* if modified, line may be wrong */
+				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
+
+			ret = navqueue_goto_line(old_doc, doc, line);
+			if (ret && ui_is_keyval_enter_or_return(keyval))
+				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+
+			return ret;
+		}
+	}
+	return FALSE;
+}
+
+
+gboolean msgwin_goto_compiler_file_line(guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	gchar *string;
-	gboolean ret = FALSE;
 	GdkColor *color;
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
@@ -586,9 +684,10 @@ gboolean msgwin_goto_compiler_file_line()
 	{
 		/* if the item is not coloured red, it's not an error line */
 		gtk_tree_model_get(model, &iter, 0, &color, -1);
-		if (! gdk_color_equal(color, &color_error))
+		if (color == NULL || ! gdk_color_equal(color, &color_error))
 		{
-			gdk_color_free(color);
+			if (color != NULL)
+				gdk_color_free(color);
 			return FALSE;
 		}
 		gdk_color_free(color);
@@ -599,41 +698,21 @@ gboolean msgwin_goto_compiler_file_line()
 			gint line;
 			gchar *filename, *dir;
 			GtkTreePath *path;
-			/* save the beginning of the string to use when free'ing it after it was stripped */
-			gchar *string_start = string;
+			gboolean ret;
 
 			path = gtk_tree_model_get_path(model, &iter);
 			find_prev_build_dir(path, model, &dir);
 			gtk_tree_path_free(path);
 			msgwin_parse_compiler_error_line(string, dir, &filename, &line);
+			g_free(string);
+			g_free(dir);
 
-			if (dir != NULL)
-				g_free(dir);
-
-			if (filename != NULL && line > -1)
-			{
-				gchar *utf8_filename = utils_get_utf8_from_locale(filename);
-				GeanyDocument *doc = document_find_by_filename(utf8_filename);
-				GeanyDocument *old_doc = document_get_current();
-
-				g_free(utf8_filename);
-
-				if (doc == NULL)	/* file not already open */
-					doc = document_open_file(filename, FALSE, NULL, NULL);
-
-				if (doc != NULL)
-				{
-					if (! doc->changed)	/* if modified, line may be wrong */
-						editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
-
-					ret = navqueue_goto_line(old_doc, doc, line);
-				}
-			}
+			ret = goto_compiler_file_line(filename, line, keyval);
 			g_free(filename);
-			g_free(string_start);
+			return ret;
 		}
 	}
-	return ret;
+	return FALSE;
 }
 
 
@@ -720,7 +799,7 @@ static void parse_compiler_error_line(const gchar *string,
 			 * Parse error: syntax error, unexpected T_LNUMBER, expecting T_FUNCTION in bob.php on line 16 */
 			gchar *tmp = strstr(string, " in ");
 
-			if(tmp != NULL)
+			if (tmp != NULL)
 			{
 				data.string = tmp;
 				data.pattern = " ";
@@ -882,7 +961,7 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 	*filename = NULL;
 	*line = -1;
 
-	if (string == NULL)
+	if (G_UNLIKELY(string == NULL))
 		return;
 
 	if (dir == NULL)
@@ -905,7 +984,7 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 }
 
 
-gboolean msgwin_goto_messages_file_line()
+gboolean msgwin_goto_messages_file_line(guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -925,6 +1004,8 @@ gboolean msgwin_goto_messages_file_line()
 		if (line >= 0 && DOC_VALID(doc))
 		{
 			ret = navqueue_goto_line(old_doc, doc, line);
+			if (ret && ui_is_keyval_enter_or_return(keyval))
+				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 		}
 		else if (line < 0 && string != NULL)
 		{
@@ -935,7 +1016,11 @@ gboolean msgwin_goto_messages_file_line()
 				/* use document_open_file to find an already open file, or open it in place */
 				doc = document_open_file(filename, FALSE, NULL, NULL);
 				if (doc != NULL)
+				{
 					ret = navqueue_goto_line(old_doc, doc, line);
+					if (ret && ui_is_keyval_enter_or_return(keyval))
+						gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+				}
 			}
 			g_free(filename);
 		}
@@ -983,12 +1068,12 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 		{
 			case MSG_COMPILER:
 			{	/* single click in the compiler treeview */
-				msgwin_goto_compiler_file_line();
+				msgwin_goto_compiler_file_line(0);
 				break;
 			}
 			case MSG_MESSAGE:
 			{	/* single click in the message treeview (results of 'Find usage') */
-				msgwin_goto_messages_file_line();
+				msgwin_goto_messages_file_line(0);
 				break;
 			}
 		}
@@ -1024,7 +1109,7 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 
 /**
  *  Switches to the given notebook tab of the messages window and shows the messages window
- *  if it was previously hidden and @c show is set to @a TRUE.
+ *  if it was previously hidden and @a show is set to @c TRUE.
  *
  *  @param tabnum An index of a tab in the messages window. Valid values are all elements of
  *                #MessageWindowTabNum.
@@ -1038,7 +1123,7 @@ void msgwin_switch_tab(gint tabnum, gboolean show)
 
 	switch (tabnum)
 	{
-		case MSG_SCRATCH: widget = ui_lookup_widget(main_widgets.window, "textview_scribble"); break;
+		case MSG_SCRATCH: widget = msgwindow.scribble; break;
 		case MSG_COMPILER: widget = msgwindow.tree_compiler; break;
 		case MSG_STATUS: widget = msgwindow.tree_status; break;
 		case MSG_MESSAGE: widget = msgwindow.tree_msg; break;
@@ -1057,11 +1142,12 @@ void msgwin_switch_tab(gint tabnum, gboolean show)
 		gtk_widget_grab_focus(widget);
 }
 
+
 /**
- *  Removes all messages from a tab specified by @c tabnum in the messages window.
+ *  Removes all messages from a tab specified by @a tabnum in the messages window.
  *
  *  @param tabnum An index of a tab in the messages window which should be cleared.
- *                Valid values are @a MSG_STATUS, @a MSG_COMPILER and @a MSG_MESSAGE.
+ *                Valid values are @c MSG_STATUS, @c MSG_COMPILER and @c MSG_MESSAGE.
  *
  * @since 0.15
  **/
