@@ -1,8 +1,8 @@
 /*
  *      project.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.c 3814 2009-05-26 19:49:18Z eht16 $
+ * $Id: project.c 4921 2010-05-16 17:43:24Z eht16 $
  */
 
 /** @file project.h
@@ -41,14 +41,13 @@
 #include "msgwindow.h"
 #include "main.h"
 #include "keyfile.h"
-#ifdef G_OS_WIN32
-# include "win32.h"
-#endif
+#include "win32.h"
 #include "build.h"
 #include "interface.h"
 #include "editor.h"
 #include "stash.h"
-#include "treeviews.h"
+#include "sidebar.h"
+#include "filetypes.h"
 
 
 ProjectPrefs project_prefs = { NULL, FALSE, FALSE };
@@ -57,12 +56,12 @@ ProjectPrefs project_prefs = { NULL, FALSE, FALSE };
 static GeanyProjectPrivate priv;
 static GeanyIndentPrefs indentation;
 
-static GeanyPrefGroup *indent_group = NULL;
+static StashGroup *indent_group = NULL;
 
 static struct
 {
 	gchar *project_file_path; /* in UTF-8 */
-} local_prefs = {NULL};
+} local_prefs = { NULL };
 
 
 static gboolean entries_modified;
@@ -75,24 +74,24 @@ typedef struct _PropertyDialogElements
 	GtkWidget *description;
 	GtkWidget *file_name;
 	GtkWidget *base_path;
-	GtkWidget *make_in_base_path;
-	GtkWidget *run_cmd;
 	GtkWidget *patterns;
+	BuildTableData  build_properties;
 } PropertyDialogElements;
 
 
 
 static gboolean update_config(const PropertyDialogElements *e);
 static void on_file_save_button_clicked(GtkButton *button, PropertyDialogElements *e);
-static void on_file_open_button_clicked(GtkButton *button, PropertyDialogElements *e);
 static gboolean load_config(const gchar *filename);
 static gboolean write_config(gboolean emit_signal);
 static void on_name_entry_changed(GtkEditable *editable, PropertyDialogElements *e);
 static void on_entries_changed(GtkEditable *editable, PropertyDialogElements *e);
+static void on_radio_long_line_custom_toggled(GtkToggleButton *radio, GtkWidget *spin_long_line);
+static void apply_editor_prefs(void);
 
 
 #define SHOW_ERR(args) dialogs_show_msgbox(GTK_MESSAGE_ERROR, args)
-#define SHOW_ERR1(args,more) dialogs_show_msgbox(GTK_MESSAGE_ERROR, args, more)
+#define SHOW_ERR1(args, more) dialogs_show_msgbox(GTK_MESSAGE_ERROR, args, more)
 #define MAX_NAME_LEN 50
 /* "projects" is part of the default project base path so be careful when translating
  * please avoid special characters and spaces, look at the source for details or ask Frank */
@@ -108,9 +107,9 @@ void project_new(void)
 	GtkWidget *bbox;
 	GtkWidget *label;
 	PropertyDialogElements *e;
-	gint response;
 
-	if (! project_ask_close()) return;
+	if (! project_ask_close())
+		return;
 
 	g_return_if_fail(app->project == NULL);
 
@@ -141,7 +140,7 @@ void project_new(void)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
 
 	e->name = gtk_entry_new();
-	ui_entry_add_clear_icon(e->name);
+	ui_entry_add_clear_icon(GTK_ENTRY(e->name));
 	gtk_entry_set_max_length(GTK_ENTRY(e->name), MAX_NAME_LEN);
 
 	ui_table_add_row(GTK_TABLE(table), 0, label, e->name, NULL);
@@ -150,7 +149,7 @@ void project_new(void)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
 
 	e->file_name = gtk_entry_new();
-	ui_entry_add_clear_icon(e->file_name);
+	ui_entry_add_clear_icon(GTK_ENTRY(e->file_name));
 	gtk_entry_set_width_chars(GTK_ENTRY(e->file_name), 30);
 	button = gtk_button_new();
 	g_signal_connect(button, "clicked", G_CALLBACK(on_file_save_button_clicked), e);
@@ -166,7 +165,7 @@ void project_new(void)
 	gtk_misc_set_alignment(GTK_MISC(label), 1, 0);
 
 	e->base_path = gtk_entry_new();
-	ui_entry_add_clear_icon(e->base_path);
+	ui_entry_add_clear_icon(GTK_ENTRY(e->base_path));
 	ui_widget_set_tooltip_text(e->base_path,
 		_("Base directory of all files that make up the project. "
 		"This can be a new path, or an existing directory tree. "
@@ -188,16 +187,14 @@ void project_new(void)
 
 	gtk_widget_show_all(e->dialog);
 
-	retry:
-	response = gtk_dialog_run(GTK_DIALOG(e->dialog));
-	if (response == GTK_RESPONSE_OK)
+	while (gtk_dialog_run(GTK_DIALOG(e->dialog)) == GTK_RESPONSE_OK)
 	{
-		if (! update_config(e))
-			goto retry;
-		else
+		if (update_config(e))
+		{
 			ui_add_recent_project_file(app->project->file_name);
+			break;
+		}
 	}
-
 	gtk_widget_destroy(e->dialog);
 	g_free(e);
 }
@@ -222,12 +219,7 @@ gboolean project_load_file_with_session(const gchar *locale_file_name)
 #ifndef G_OS_WIN32
 static void run_open_dialog(GtkDialog *dialog)
 {
-	gint response;
-
-	retry:
-	response = gtk_dialog_run(dialog);
-
-	if (response == GTK_RESPONSE_ACCEPT)
+	while (gtk_dialog_run(dialog) == GTK_RESPONSE_ACCEPT)
 	{
 		gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
 
@@ -240,9 +232,10 @@ static void run_open_dialog(GtkDialog *dialog)
 			gtk_widget_grab_focus(GTK_WIDGET(dialog));
 			g_free(utf8_filename);
 			g_free(filename);
-			goto retry;
+			continue;
 		}
 		g_free(filename);
+		break;
 	}
 }
 #endif
@@ -316,9 +309,25 @@ void project_open(void)
 /* Called when creating, opening, closing and updating projects. */
 static void update_ui(void)
 {
+	if (main_status.quitting)
+		return;
+
 	ui_set_window_title(NULL);
 	build_menu_update(NULL);
-	treeviews_openfiles_update_all();
+	sidebar_openfiles_update_all();
+}
+
+
+static void remove_foreach_project_filetype(gpointer data, gpointer user_data)
+{
+	GeanyFiletype *ft = (GeanyFiletype*)data;
+	if (ft != NULL)
+	{
+		setptr(ft->projfilecmds, NULL);
+		setptr(ft->projexeccmds, NULL);
+		setptr(ft->projerror_regex_string, NULL);
+		ft->project_list_entry = -1;
+	}
 }
 
 
@@ -332,15 +341,27 @@ void project_close(gboolean open_default)
 	/* use write_config() to save project session files */
 	write_config(FALSE);
 
+	/* remove project filetypes build entries */
+	if (app->project->build_filetypes_list != NULL)
+	{
+		g_ptr_array_foreach(app->project->build_filetypes_list, remove_foreach_project_filetype, NULL);
+		g_ptr_array_free(app->project->build_filetypes_list, FALSE);
+	}
+
+	/* remove project non filetype build menu items */
+	build_remove_menu_item(GEANY_BCS_PROJ, GEANY_GBG_NON_FT, -1);
+	build_remove_menu_item(GEANY_BCS_PROJ, GEANY_GBG_EXEC, -1);
+
 	g_free(app->project->name);
 	g_free(app->project->description);
 	g_free(app->project->file_name);
 	g_free(app->project->base_path);
-	g_free(app->project->run_cmd);
 
 	g_free(app->project);
 	app->project = NULL;
 
+	apply_editor_prefs(); /* ensure that global settings are restored */
+
 	if (project_prefs.project_session)
 	{
 		/* close all existing tabs first */
@@ -361,22 +382,28 @@ void project_close(gboolean open_default)
 }
 
 
+static void on_set_use_base_path_clicked(GtkWidget *unused1, gpointer user_data)
+{
+	build_set_non_ft_wd_to_proj((BuildTableData)user_data);
+}
+
+
 static void create_properties_dialog(PropertyDialogElements *e)
 {
-	GtkWidget *table, *notebook;
-	GtkWidget *image;
+	GtkWidget *table, *notebook, *build_table;
 	GtkWidget *button;
 	GtkWidget *bbox;
 	GtkWidget *label;
 	GtkWidget *swin;
+	GeanyDocument *doc = document_get_current();
+	GeanyFiletype *ft = NULL;
 
 	e->dialog = create_project_dialog();
 	gtk_window_set_transient_for(GTK_WINDOW(e->dialog), GTK_WINDOW(main_widgets.window));
 	gtk_window_set_destroy_with_parent(GTK_WINDOW(e->dialog), TRUE);
 	gtk_widget_set_name(e->dialog, "GeanyDialogProject");
 
-	ui_entry_add_clear_icon(ui_lookup_widget(e->dialog, "spin_indent_width"));
-	ui_entry_add_clear_icon(ui_lookup_widget(e->dialog, "spin_tab_width"));
+	ui_entry_add_clear_icon(GTK_ENTRY(ui_lookup_widget(e->dialog, "spin_indent_width")));
 
 	table = gtk_table_new(6, 2, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 6);
@@ -387,10 +414,10 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
 	e->name = gtk_entry_new();
-	ui_entry_add_clear_icon(e->name);
+	ui_entry_add_clear_icon(GTK_ENTRY(e->name));
 	gtk_entry_set_max_length(GTK_ENTRY(e->name), MAX_NAME_LEN);
 	gtk_table_attach(GTK_TABLE(table), e->name, 1, 2, 0, 1,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -400,10 +427,10 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
 	e->file_name = gtk_entry_new();
-	ui_entry_add_clear_icon(e->file_name);
+	ui_entry_add_clear_icon(GTK_ENTRY(e->file_name));
 	gtk_editable_set_editable(GTK_EDITABLE(e->file_name), FALSE);	/* read-only */
 	gtk_table_attach(GTK_TABLE(table), e->file_name, 1, 2, 1, 2,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
@@ -413,7 +440,7 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
 	e->description = gtk_text_view_new();
 	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(e->description), GTK_WRAP_WORD);
@@ -430,10 +457,10 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
 	e->base_path = gtk_entry_new();
-	ui_entry_add_clear_icon(e->base_path);
+	ui_entry_add_clear_icon(GTK_ENTRY(e->base_path));
 	ui_widget_set_tooltip_text(e->base_path,
 		_("Base directory of all files that make up the project. "
 		"This can be a new path, or an existing directory tree. "
@@ -444,33 +471,31 @@ static void create_properties_dialog(PropertyDialogElements *e)
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 
-	e->make_in_base_path = gtk_check_button_new_with_label(_("Make in base path"));
-	gtk_table_attach(GTK_TABLE(table), e->make_in_base_path, 0, 3, 4, 5,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
+	if (doc != NULL) ft = doc->file_type;
+	build_table = build_commands_table(doc, GEANY_BCS_PROJ, &(e->build_properties), ft);
+	gtk_container_set_border_width(GTK_CONTAINER(build_table), 6);
+	label = gtk_label_new(_("Build"));
+	notebook = ui_lookup_widget(e->dialog, "project_notebook");
+	gtk_notebook_insert_page(GTK_NOTEBOOK(notebook), build_table, label, 2);
 
-	label = gtk_label_new(_("Run command:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
+	label = gtk_label_new(_("Set the Build non-filetype working directories to use base path:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
-	e->run_cmd = gtk_entry_new();
-	ui_entry_add_clear_icon(e->run_cmd);
-	ui_widget_set_tooltip_text(e->run_cmd,
-		_("Command-line to run in the project base directory. "
-		"Options can be appended to the command. "
-		"Leave blank to use the default run command."));
-	button = gtk_button_new();
-	g_signal_connect(button, "clicked", G_CALLBACK(on_file_open_button_clicked), e);
-	image = gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_BUTTON);
-	gtk_container_add(GTK_CONTAINER(button), image);
+	button = gtk_button_new_with_label(_("Set"));
+	ui_widget_set_tooltip_text(button,
+		_("Set the working directories (on the Build tab) "
+		"for the non-filetype build commands to use the base path"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+	g_signal_connect(button, "clicked", G_CALLBACK(on_set_use_base_path_clicked), e->build_properties);
 	bbox = gtk_hbox_new(FALSE, 6);
-	gtk_box_pack_start_defaults(GTK_BOX(bbox), e->run_cmd);
+	gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-	gtk_table_attach(GTK_TABLE(table), bbox, 1, 2, 5, 6,
+	gtk_table_attach(GTK_TABLE(table), bbox, 0, 2, 4, 5,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
+					(GtkAttachOptions) (GTK_FILL), 0, 0);
+
+	g_signal_connect(ui_lookup_widget(e->dialog, "radio_long_line_custom"), "toggled",
+		G_CALLBACK(on_radio_long_line_custom_toggled), ui_lookup_widget(e->dialog, "spin_long_line"));
 
 #if 0
 	label = gtk_label_new(_("File patterns:"));
@@ -478,7 +503,7 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), -1, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
 	e->patterns = gtk_text_view_new();
 	swin = gtk_scrolled_window_new(NULL, NULL);
@@ -491,7 +516,6 @@ static void create_properties_dialog(PropertyDialogElements *e)
 					(GtkAttachOptions) (0), 0, 0);
 #endif
 
-	notebook = ui_lookup_widget(e->dialog, "project_notebook");
 	label = gtk_label_new(_("Project"));
 	gtk_widget_show(table);	/* needed to switch current page */
 	gtk_notebook_insert_page(GTK_NOTEBOOK(notebook), table, label, 0);
@@ -503,7 +527,8 @@ void project_properties(void)
 {
 	PropertyDialogElements *e = g_new(PropertyDialogElements, 1);
 	GeanyProject *p = app->project;
-	gint response;
+	GtkWidget *widget = NULL;
+	GtkWidget *radio_long_line_custom;
 
 	g_return_if_fail(app->project != NULL);
 
@@ -515,6 +540,21 @@ void project_properties(void)
 
 	/* fill the elements with the appropriate data */
 	gtk_entry_set_text(GTK_ENTRY(e->name), p->name);
+	gtk_entry_set_text(GTK_ENTRY(e->file_name), p->file_name);
+	gtk_entry_set_text(GTK_ENTRY(e->base_path), p->base_path);
+
+	radio_long_line_custom = ui_lookup_widget(e->dialog, "radio_long_line_custom");
+	switch (p->long_line_behaviour)
+	{
+		case 0: widget = ui_lookup_widget(e->dialog, "radio_long_line_disabled"); break;
+		case 1: widget = ui_lookup_widget(e->dialog, "radio_long_line_default"); break;
+		case 2: widget = radio_long_line_custom; break;
+	}
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
+
+	widget = ui_lookup_widget(e->dialog, "spin_long_line");
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), (gdouble)p->long_line_column);
+	on_radio_long_line_custom_toggled(GTK_TOGGLE_BUTTON(radio_long_line_custom), widget);
 
 	if (p->description != NULL)
 	{	/* set text */
@@ -543,25 +583,17 @@ void project_properties(void)
 	}
 #endif
 
-	gtk_entry_set_text(GTK_ENTRY(e->file_name), p->file_name);
-	gtk_entry_set_text(GTK_ENTRY(e->base_path), p->base_path);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(e->make_in_base_path),
-		p->make_in_base_path);
-	if (p->run_cmd != NULL)
-		gtk_entry_set_text(GTK_ENTRY(e->run_cmd), p->run_cmd);
-
 	gtk_widget_show_all(e->dialog);
 
-	retry:
-	response = gtk_dialog_run(GTK_DIALOG(e->dialog));
-	if (response == GTK_RESPONSE_OK)
+	while (gtk_dialog_run(GTK_DIALOG(e->dialog)) == GTK_RESPONSE_OK)
 	{
-		if (! update_config(e))
-			goto retry;
-
-		stash_group_update(indent_group, e->dialog);
+		if (update_config(e))
+		{
+			stash_group_update(indent_group, e->dialog);
+			break;
+		}
 	}
-
+	build_free_fields(e->build_properties);
 	gtk_widget_destroy(e->dialog);
 	g_free(e);
 }
@@ -598,6 +630,9 @@ static GeanyProject *create_project(void)
 	priv.indentation = &indentation;
 	project->priv = &priv;
 
+	project->long_line_behaviour = 1 /* use global settings */;
+	project->long_line_column = editor_prefs.long_line_global_column;
+
 	app->project = project;
 	return project;
 }
@@ -701,12 +736,11 @@ static gboolean update_config(const PropertyDialogElements *e)
 	if (! new_project)	/* save properties specific fields */
 	{
 		GtkTextIter start, end;
-		/*gchar *tmp;*/
 		GtkTextBuffer *buffer;
-
-		p->make_in_base_path = gtk_toggle_button_get_active(
-			GTK_TOGGLE_BUTTON(e->make_in_base_path));
-		setptr(p->run_cmd, g_strdup(gtk_entry_get_text(GTK_ENTRY(e->run_cmd))));
+		GeanyDocument *doc = document_get_current();
+		GeanyBuildCommand *oldvalue;
+		GeanyFiletype *ft = doc ? doc->file_type : NULL;
+		GtkWidget *widget;
 
 		/* get and set the project description */
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(e->description));
@@ -714,6 +748,36 @@ static gboolean update_config(const PropertyDialogElements *e)
 		gtk_text_buffer_get_end_iter(buffer, &end);
 		setptr(p->description, g_strdup(gtk_text_buffer_get_text(buffer, &start, &end, FALSE)));
 
+		/* read the project build menu */
+		oldvalue = ft ? ft->projfilecmds : NULL;
+		build_read_project(ft, e->build_properties);
+
+		if (ft != NULL && ft->projfilecmds != oldvalue && ft->project_list_entry < 0)
+		{
+			if (p->build_filetypes_list == NULL)
+				p->build_filetypes_list = g_ptr_array_new();
+			ft->project_list_entry = p->build_filetypes_list->len;
+			g_ptr_array_add(p->build_filetypes_list, ft);
+		}
+		build_menu_update(doc);
+
+		widget = ui_lookup_widget(e->dialog, "radio_long_line_disabled");
+		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+			p->long_line_behaviour = 0;
+		else
+		{
+			widget = ui_lookup_widget(e->dialog, "radio_long_line_default");
+			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
+				p->long_line_behaviour = 1;
+			else
+				/* "Custom" radio button must be checked */
+				p->long_line_behaviour = 2;
+		}
+
+		widget = ui_lookup_widget(e->dialog, "spin_long_line");
+		p->long_line_column = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		apply_editor_prefs();
+
 #if 0
 		/* get and set the project file patterns */
 		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(e->patterns));
@@ -769,8 +833,7 @@ static void run_dialog(GtkWidget *dialog, GtkWidget *entry)
 			g_free(locale_dir);
 		}
 	}
-	else
-	if (gtk_file_chooser_get_action(GTK_FILE_CHOOSER(dialog)) != GTK_FILE_CHOOSER_ACTION_OPEN)
+	else if (gtk_file_chooser_get_action(GTK_FILE_CHOOSER(dialog)) != GTK_FILE_CHOOSER_ACTION_OPEN)
 	{
 		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), utf8_filename);
 	}
@@ -821,35 +884,6 @@ static void on_file_save_button_clicked(GtkButton *button, PropertyDialogElement
 }
 
 
-static void on_file_open_button_clicked(GtkButton *button, PropertyDialogElements *e)
-{
-#ifdef G_OS_WIN32
-	gchar *path = win32_show_project_open_dialog(e->dialog, _("Choose Project Run Command"),
-						gtk_entry_get_text(GTK_ENTRY(e->run_cmd)), FALSE, FALSE);
-	if (path != NULL)
-	{
-		gtk_entry_set_text(GTK_ENTRY(e->run_cmd), path);
-		g_free(path);
-	}
-#else
-	GtkWidget *dialog;
-
-	/* initialise the dialog */
-	dialog = gtk_file_chooser_dialog_new(_("Choose Project Run Command"), NULL,
-					GTK_FILE_CHOOSER_ACTION_OPEN,
-					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-	gtk_widget_set_name(dialog, "GeanyDialog");
-	gtk_window_set_destroy_with_parent(GTK_WINDOW(dialog), TRUE);
-	gtk_window_set_skip_taskbar_hint(GTK_WINDOW(dialog), TRUE);
-	gtk_window_set_type_hint(GTK_WINDOW(dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
-	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT);
-
-	run_dialog(dialog, e->run_cmd);
-#endif
-}
-
-
 /* sets the project base path and the project file name according to the project name */
 static void on_name_entry_changed(GtkEditable *editable, PropertyDialogElements *e)
 {
@@ -862,7 +896,7 @@ static void on_name_entry_changed(GtkEditable *editable, PropertyDialogElements
 		return;
 
 	name = gtk_editable_get_chars(editable, 0, -1);
-	if (name != NULL && strlen(name) > 0)
+	if (NZV(name))
 	{
 		base_path = g_strconcat(project_dir, G_DIR_SEPARATOR_S,
 			name, G_DIR_SEPARATOR_S, NULL);
@@ -896,6 +930,12 @@ static void on_entries_changed(GtkEditable *editable, PropertyDialogElements *e)
 }
 
 
+static void on_radio_long_line_custom_toggled(GtkToggleButton *radio, GtkWidget *spin_long_line)
+{
+	gtk_widget_set_sensitive(spin_long_line, gtk_toggle_button_get_active(radio));
+}
+
+
 gboolean project_load_file(const gchar *locale_file_name)
 {
 	g_return_val_if_fail(locale_file_name != NULL, FALSE);
@@ -948,10 +988,15 @@ static gboolean load_config(const gchar *filename)
 	p->description = utils_get_setting_string(config, "project", "description", "");
 	p->file_name = utils_get_utf8_from_locale(filename);
 	p->base_path = utils_get_setting_string(config, "project", "base_path", "");
-	p->make_in_base_path = utils_get_setting_boolean(config, "project", "make_in_base_path", TRUE);
-	p->run_cmd = utils_get_setting_string(config, "project", "run_cmd", "");
 	p->file_patterns = g_key_file_get_string_list(config, "project", "file_patterns", NULL, NULL);
 
+	p->long_line_behaviour = utils_get_setting_integer(config, "long line marker",
+		"long_line_behaviour", 1 /* follow global */);
+	p->long_line_column = utils_get_setting_integer(config, "long line marker",
+		"long_line_column", editor_prefs.long_line_global_column);
+	apply_editor_prefs();
+
+	build_load_menu(config, GEANY_BCS_PROJ, (gpointer)p);
 	if (project_prefs.project_session)
 	{
 		/* save current (non-project) session (it could has been changed since program startup) */
@@ -969,6 +1014,15 @@ static gboolean load_config(const gchar *filename)
 }
 
 
+static void apply_editor_prefs(void)
+{
+	guint i;
+
+	foreach_document(i)
+		editor_apply_update_prefs(documents[i]->editor);
+}
+
+
 /* Write the project settings as well as the project session files into its configuration files.
  * emit_signal defines whether the project-save signal should be emitted. When write_config()
  * is called while closing a project, this is used to skip emitting the signal because
@@ -998,17 +1052,17 @@ static gboolean write_config(gboolean emit_signal)
 
 	if (p->description)
 		g_key_file_set_string(config, "project", "description", p->description);
-	g_key_file_set_boolean(config, "project", "make_in_base_path", p->make_in_base_path);
-	if (p->run_cmd)
-		g_key_file_set_string(config, "project", "run_cmd", p->run_cmd);
 	if (p->file_patterns)
 		g_key_file_set_string_list(config, "project", "file_patterns",
 			(const gchar**) p->file_patterns, g_strv_length(p->file_patterns));
 
+	g_key_file_set_integer(config, "long line marker", "long_line_behaviour", p->long_line_behaviour);
+	g_key_file_set_integer(config, "long line marker", "long_line_column", p->long_line_column);
+
 	/* store the session files into the project too */
 	if (project_prefs.project_session)
 		configuration_save_session_files(config);
-
+	build_save_menu(config, (gpointer)p, GEANY_BCS_PROJ);
 	if (emit_signal)
 	{
 		g_signal_emit_by_name(geany_object, "project-save", config);
@@ -1055,19 +1109,6 @@ gchar *project_get_base_path(void)
 }
 
 
-/* Returns: NULL if the default path should be used, or a UTF-8 path.
- * Maybe in future this will support a separate project make path from base path. */
-gchar *project_get_make_dir(void)
-{
-	GeanyProject *project = app->project;
-
-	if (project && ! project->make_in_base_path)
-		return NULL;
-	else
-		return project_get_base_path();
-}
-
-
 /* This is to save project-related global settings, NOT project file settings. */
 void project_save_prefs(GKeyFile *config)
 {
@@ -1075,7 +1116,7 @@ void project_save_prefs(GKeyFile *config)
 
 	if (cl_options.load_session)
 	{
-		gchar *utf8_filename = (project == NULL) ? "" : project->file_name;
+		const gchar *utf8_filename = (project == NULL) ? "" : project->file_name;
 
 		g_key_file_set_string(config, "project", "session_file", utf8_filename);
 	}
@@ -1110,7 +1151,7 @@ void project_setup_prefs(void)
 	static gboolean callback_setup = FALSE;
 
 	g_return_if_fail(local_prefs.project_file_path != NULL);
-	
+
 	gtk_entry_set_text(GTK_ENTRY(path_entry), local_prefs.project_file_path);
 	if (! callback_setup)
 	{	/* connect the callback only once */
@@ -1134,7 +1175,7 @@ void project_apply_prefs(void)
 
 void project_init(void)
 {
-	GeanyPrefGroup *group;
+	StashGroup *group;
 
 	group = stash_group_new("indentation");
 	/* defaults are copied from editor indent prefs */
@@ -1149,8 +1190,9 @@ void project_init(void)
 		"radio_indent_tabs", GEANY_INDENT_TYPE_TABS,
 		"radio_indent_both", GEANY_INDENT_TYPE_BOTH,
 		NULL);
-	stash_group_add_spin_button_integer(group, &indentation.hard_tab_width,
-		"indent_hard_tab_width", 8, "spin_tab_width");
+	/* This is a 'hidden' pref for backwards-compatibility */
+	stash_group_add_integer(group, &indentation.hard_tab_width,
+		"indent_hard_tab_width", 8);
 	stash_group_add_toggle_button(group, &indentation.detect_type,
 		"detect_indent", FALSE, "check_detect_indent");
 	stash_group_add_combo_box(group, (gint*)(gpointer)&indentation.auto_indent_mode,
@@ -1162,4 +1204,3 @@ void project_finalize(void)
 {
 	stash_group_free(indent_group);
 }
-
