@@ -1,8 +1,8 @@
 /*
  *      project.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.h 3446 2009-01-04 18:30:42Z eht16 $
+ * $Id: project.h 4751 2010-03-10 16:20:34Z ntrel $
  */
 
 
@@ -35,14 +35,18 @@ typedef struct GeanyProject
 	gchar *description; 	/**< Short description of the project. */
 	gchar *file_name; 		/**< Where the project file is stored (in UTF-8). */
 	gchar *base_path;		/**< Base path of the project directory (in UTF-8, maybe relative). */
-	gchar *run_cmd; 		/**< Project run command (in UTF-8). */
 	/** Identifier whether it is a pure Geany project or modified/extended
 	 * by a plugin. */
 	gint type;
 	gchar **file_patterns;	/**< Array of filename extension patterns. */
-	gboolean make_in_base_path;
 
 	struct GeanyProjectPrivate	*priv;	/* must be last, append fields before this item */
+#ifdef GEANY_PRIVATE
+	/* Do not use following fields in plugins */
+	GPtrArray *build_filetypes_list; /* Project has custom filetype builds for these. */
+	gint long_line_behaviour; /* 0 - disabled, 1 - follow global settings, 2 - enabled (custom) */
+	gint long_line_column; /* Long line marker position. */
+#endif
 }
 GeanyProject;
 
@@ -70,12 +74,14 @@ void project_close(gboolean open_default);
 
 void project_properties(void);
 
+gboolean project_ask_close(void);
+
 
 gboolean project_load_file(const gchar *locale_file_name);
 
-gchar *project_get_base_path(void);
+gboolean project_load_file_with_session(const gchar *locale_file_name);
 
-gchar *project_get_make_dir(void);
+gchar *project_get_base_path(void);
 
 
 void project_save_prefs(GKeyFile *config);
