@@ -1,8 +1,8 @@
 /*
  *      tools.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
- *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,20 +18,19 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: tools.h 1751 2007-07-27 11:28:17Z ntrel $
+ * $Id: tools.h 4518 2010-01-01 22:55:18Z eht16 $
  */
 
 
 #ifndef GEANY_TOOLS_H
 #define GEANY_TOOLS_H 1
 
-void tools_create_insert_custom_command_menu_items();
+void tools_create_insert_custom_command_menu_items(void);
 
-void tools_execute_custom_command(gint idx, const gchar *command);
+void tools_execute_custom_command(GeanyDocument *doc, const gchar *command);
 
+void tools_word_count(void);
 
-void tools_word_count();
-
-void tools_color_chooser(gchar *color);
+void tools_color_chooser(const gchar *color);
 
 #endif
