@@ -1,8 +1,8 @@
 /*
  *      utils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.h 3524 2009-01-28 19:30:18Z eht16 $
+ * $Id: utils.h 4760 2010-03-15 13:01:16Z ntrel $
  */
 
 /**
@@ -29,12 +29,16 @@
 #ifndef GEANY_UTILS_H
 #define GEANY_UTILS_H 1
 
-/** Returns: TRUE if @a ptr points to a non-zero value. */
+
+#include <time.h>
+
+
+/** Returns TRUE if @a ptr points to a non-zero value. */
 #define NZV(ptr) \
-	((ptr) && (ptr)[0])
+	(G_LIKELY((ptr)) && G_LIKELY((ptr)[0]))
 
 /**
- *  Free's @a ptr (if not @c NULL), then assigns @a result to it.
+ *  Frees @a ptr (if not @c NULL), then assigns @a result to it.
  *  @a result can be an expression using the 'old' value of @a ptr.
  *  It prevents a memory leak compared with: @code ptr = func(ptr); @endcode
  **/
@@ -45,13 +49,63 @@
 		g_free(setptr_tmp);\
 	}
 
+/** Duplicates a string on the stack using @c g_alloca().
+ * Like glibc's @c strdupa(), but portable.
+ * @note You must include @c string.h yourself.
+ * @warning Don't use excessively or for long strings otherwise there may be stack exhaustion -
+ *          see the GLib docs for @c g_alloca(). */
+#define utils_strdupa(str) \
+	strcpy(g_alloca(strlen(str) + 1), str)
+
+/** Iterates all the items in @a array using pointers.
+ * @param item pointer to an item in @a array.
+ * @param array C array to traverse.
+ * @param len Length of the array. */
 #define foreach_c_array(item, array, len) \
 	for (item = array; item < &array[len]; item++)
 
-/* @param ptr should be a (gpointer*), needed for implementation. */
-#define foreach_ptr_array(item, ptr, ptr_array) \
-	for (ptr = ptr_array->pdata, item = *ptr; \
-		ptr < &ptr_array->pdata[ptr_array->len]; ++ptr, item = *ptr)
+/** Iterates all items in @a array.
+ * @param type Type of @a item.
+ * @param item pointer to item in @a array.
+ * @param array @c GArray to traverse. */
+#define foreach_array(type, item, array) \
+	foreach_c_array(item, ((type*)(gpointer)array->data), array->len)
+
+/** Iterates all the pointers in @a ptr_array.
+ * @param item pointer in @a ptr_array.
+ * @param idx @c guint index into @a ptr_array.
+ * @param ptr_array @c GPtrArray to traverse. */
+#define foreach_ptr_array(item, idx, ptr_array) \
+	for (idx = 0, item = g_ptr_array_index(ptr_array, 0); \
+		idx < ptr_array->len; ++idx, item = g_ptr_array_index(ptr_array, idx))
+
+/** Iterates all the nodes in @a list.
+ * @param node should be a (@c GList*).
+ * @param list @c GList to traverse. */
+#define foreach_list(node, list) \
+	for (node = list; node != NULL; node = node->next)
+
+/** Iterates all the nodes in @a list.
+ * @param node should be a (@c GSList*).
+ * @param list @c GSList to traverse. */
+#define foreach_slist(node, list) \
+	foreach_list(node, list)
+
+/** Iterates through each unsorted filename in a @c GDir.
+ * @param filename (@c const @c gchar*) locale-encoded filename, without path. Do not modify or free.
+ * @param dir @c GDir created with @c g_dir_open(). Call @c g_dir_close() afterwards.
+ * @see utils_get_file_list() if you want a sorted list.
+ * @since Geany 0.19. */
+#define foreach_dir(filename, dir)\
+	for ((filename) = g_dir_read_name(dir); (filename) != NULL; (filename) = g_dir_read_name(dir))
+
+/** Iterates through each character in @a string.
+ * @param char_ptr Pointer to character.
+ * @param string String to traverse.
+ * @warning Doesn't include null terminating character.
+ * @since Geany 0.19. */
+#define foreach_str(char_ptr, string) \
+	for (char_ptr = string; *char_ptr; char_ptr++)
 
 
 void utils_open_browser(const gchar *uri);
@@ -70,8 +124,12 @@ const gchar *utils_get_eol_name(gint eol_mode);
 
 gboolean utils_atob(const gchar *str);
 
+void utils_tidy_path(gchar *filename);
+
 gboolean utils_is_absolute_path(const gchar *path);
 
+const gchar *utils_path_skip_root(const gchar *path);
+
 gdouble utils_scale_round(gdouble val, gdouble factor);
 
 gboolean utils_str_equal(const gchar *a, const gchar *b);
@@ -86,7 +144,7 @@ guint utils_string_replace_all(GString *haystack, const gchar *needle, const gch
 
 guint utils_string_replace_first(GString *haystack, const gchar *needle, const gchar *replace);
 
-gchar *utils_str_replace(gchar *haystack, const gchar *needle, const gchar *replacement);
+void utils_str_replace_all(gchar **haystack, const gchar *needle, const gchar *replacement);
 
 gint utils_strpos(const gchar* haystack, const gchar *needle);
 
@@ -102,6 +160,8 @@ gchar *utils_get_setting_string(GKeyFile *config, const gchar *section, const gc
 
 gchar *utils_get_hex_from_color(GdkColor *color);
 
+guint utils_invert_color(guint color);
+
 const gchar *utils_get_default_dir_utf8(void);
 
 gchar *utils_get_current_file_dir_utf8(void);
@@ -120,7 +180,7 @@ GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, gboolean nblock,
 
 gchar **utils_read_file_in_array(const gchar *filename);
 
-gboolean utils_str_replace_escape(gchar *string);
+gboolean utils_str_replace_escape(gchar *string, gboolean keep_backslash);
 
 gboolean utils_wrap_string(gchar *string, gint wrapstart);
 
@@ -136,6 +196,12 @@ gint utils_mkdir(const gchar *path, gboolean create_parent_dirs);
 
 GSList *utils_get_file_list(const gchar *path, guint *length, GError **error);
 
+GSList *utils_get_file_list_full(const gchar *path, gboolean full_path, gboolean sort, GError **error);
+
+GSList *utils_get_config_files(const gchar *subdir);
+
+gchar *utils_get_help_url(const gchar *suffix);
+
 gboolean utils_str_has_upper(const gchar *str);
 
 gint utils_is_file_writeable(const gchar *locale_filename);
@@ -159,4 +225,10 @@ gboolean utils_is_uri(const gchar *uri);
 
 gboolean utils_is_remote_path(const gchar *path);
 
+gchar *utils_str_middle_truncate(const gchar *string, guint truncate_length);
+
+gchar *utils_str_remove_chars(gchar *string, const gchar *chars);
+
+gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_varname, ...) G_GNUC_NULL_TERMINATED;
+
 #endif
